#include <iostream>
#include <stdlib.h>
#include "stdio.h"
#include "CandidateSolution.h"
#include "GeneticAlgorithm.h"
#include "FitnessResolver.h"
#include "Maze.h"
#include "MutationResolver.h"
#include "ParentResolver.h"
#include "SelectionResolver.h"
#include "SpringResolver.h"
#include <stdexcept>

GeneticAlgorithm::GeneticAlgorithm(int populationSize, int mazeSize,
		int iterationCount, int maxParentsPerIteration, int maxMutationsPerIteration,
		int chanceOfMutation) {

	if (chanceOfMutation > 100 || chanceOfMutation < 0) {
		throw std::domain_error("Chance of mutation has to be in [0, 100]");
	}

	this->populationSize = populationSize;
	this->mazeSize = mazeSize;
	this->iterationCount = iterationCount;
	this->maxParentCountPerIteration = maxParentsPerIteration;
	this->maxMutationCountPerSpring = maxMutationsPerIteration;
	this->chanceOfMutation = chanceOfMutation;
}

int GeneticAlgorithm::main() {
	srand (time(NULL));
	population = createInitialPopulation(populationSize, mazeSize);
	std::pair<CandidateSolution, CandidateSolution> parents;
	std::pair<Maze, Maze> spring;

	// main iteration loop
	for (unsigned int i = 0; i < iterationCount; i += 1) {
		std::cout << "Iteration " << i << "\n ---- \n";
		if (i==0) {
			std::cout << "Initial population: " << i << "\n ---- \n";
			for (std::set<CandidateSolution>::iterator it = population.begin(); it != population.end(); ++it) {
				CandidateSolution c = *it;
				c.getMaze().print();
				std::cout << "Fitness: " << c.getFitness() << "\n **** \n";
			}
		}
		for (unsigned int j = 0; j < maxParentCountPerIteration; j += 1) {
			parents = ParentResolver::getPairOfParents(population);
			std::pair<Maze,Maze> parentMazes(parents.first.getMaze(), parents.second.getMaze());
			spring = SpringResolver::createSpring(parentMazes);
			Maze m1 = spring.first;
			Maze m2 = spring.second;
			if ((unsigned int) rand() % 100 <= chanceOfMutation) {
				MutationResolver::mutate(m1, maxMutationCountPerSpring);
			}
			if ((unsigned int)rand() % 100 <= chanceOfMutation) {
				MutationResolver::mutate(m2, maxMutationCountPerSpring);
			}
			double f1 = FitnessResolver::getFitness(m1);
			double f2 = FitnessResolver::getFitness(m2);

			CandidateSolution c1 = CandidateSolution(i, m1, f1);
			CandidateSolution c2 = CandidateSolution(i, m2, f2);

			population.insert(c1);
			logger.logCandidateBirth(i, c1, parents.first, parents.second);
			population.insert(c2);
			logger.logCandidateBirth(i, c2, parents.first, parents.second);

			std::cout << "\nNew spring:\n ---- \n";
			spring.first.print();
			std::cout << "Fitness: " << f1 << "\n";
			spring.second.print();
			std::cout << "Fitness: " << f2 << "\n";
		}
		SelectionResolver::elitism(population, populationSize);
	}
	return 0;
}

std::set<CandidateSolution> GeneticAlgorithm::createInitialPopulation(int populationSize, int mazeSize) {
	std::set<CandidateSolution> population;
	Maze m;
	CandidateSolution c;
	double fitness;
	for (int i = 0; i < populationSize; i += 1) {
		m = Maze::createRandom(mazeSize, mazeSize);
		fitness = FitnessResolver::getFitness(m);
		c = CandidateSolution(0, m, fitness);
		logger.logCandidateBirth(0, c);
		population.insert(c);
	}
	return population;
}

