/*
 * Graph.h
 *
 *  Created on: Jul 22, 2014
 *      Author: s10z
 */

#include <set>
#include <list>
#include <map>
#include "Vertex.h"
#include "Edge.h"

#ifndef GRAPH_H_
#define GRAPH_H_

class Graph {
public:
	Graph();
	virtual ~Graph();
	bool hasVertex(Vertex vertex);
	void addVertex(Vertex vertex);
	void removeVertex(Vertex vertex);
	std::set<Vertex> getVertices();

	std::set<Vertex> getAdjacentVertices(Vertex v);

	/**
	 * Edges are a bidirectional relation between two vertices.
	 * Therefore by adding (u,v), (v,u) will be inserted as well.
	 * In case the weight is <=0, the edge will be removed instead.
	 */
	void addEdge(Vertex u, Vertex v, int weight);

	/**
	 * Removed the bidirectional relation of u,v.
	 */
	void removeEdge(Vertex u, Vertex v);

	/**
	 * Predicate to check if there is a weighted edge between
	 * the given vertices.
	 */
	bool hasEdge(Vertex u, Vertex v);

	/**
	 * In case the edge does not exist, its weight is zero.
	 */
	int getEdgeWeight(Vertex u, Vertex v);

	int getEdgeCount() const;

	int getVertexCount() const;
private:
	std::set<Vertex> vertices;
	std::map<Edge, int> edges;
	std::map<Vertex, std::set<Vertex> > adjacent;
	void setAdjacent(Vertex u, Vertex v);
	void removeAdjacent(Vertex u, Vertex v);
};

#endif /* GRAPH_H_ */
