#include "Maze.h"
#include <iostream>
#include <stdlib.h>
#include <stdexcept>
#include <stdio.h>

Maze::Maze() {
	width = 0;
	height = 0;
	fieldCount = 0;
}

Maze::Maze(unsigned int width, unsigned int height) {
	this->width = width;
	this->height = height;
	this->fieldCount = width * height;
	for (unsigned int i = 0; i < this->fieldCount; i += 1) {
		fields.push_back(false);
	}
}

Maze Maze::createRandom(unsigned int width, unsigned int height) {
	Maze m = Maze();
	m.width = width;
	m.height = height;
	m.fieldCount = width * height;
	for (unsigned int i=0; i < m.fieldCount; i+=1) {
		int value = rand() % 2;
		m.fields.push_back(0==value);
	}
	return m;
}


int Maze::getWidth() const {
	return width;
}

int Maze::getHeight() const {
	return height;
}

int Maze::getFieldCount() const {
	return fieldCount;
}

bool Maze::getFieldValue(unsigned int fieldNo) const {
	if (fieldNo >= fieldCount) {
		throw std::out_of_range("Tried to read field outside of maze");
	}
	return fields.at(fieldNo);
}

void Maze::getFieldValues(bool * values){
	for (int i = 0; i < getFieldCount(); i += 1) {
		values[i] = getFieldValue(i);
	}
}

void Maze::setFieldValue(unsigned int fieldNo, bool value) {
	if (fieldNo >= fieldCount) {
		throw std::out_of_range("Tried to mutate field outside of maze");
	}
	fields.at(fieldNo) = value;
}

void Maze::print() const {
	char representation;
	std::cout << "\nWalls | Paths\n------------- \n";

	for (unsigned int i=0; i < height; i+=1) { // log per row
		int offset = i * width;
		std::cout << '|';
		for (unsigned int j = 0; j < width; j += 1) {
			int fieldNo = offset + j;
			representation = getFieldValue(fieldNo) ? 'x' : ' ';
			std::cout << representation;
		}
		std::cout << "|    |";
		for (unsigned int j = 0; j < width; j += 1) {
			int fieldNo = offset + j;
			representation = getFieldValue(fieldNo) ? ' ' : 'o';
			std::cout << representation;
		}
		std::cout << "|\n";
	}

}

std::string Maze::toString() const {
	std::string result;
	for (std::deque<bool>::const_iterator it = fields.begin(); it != fields.end(); ++it) {
		bool value = *it;
		result.push_back(value? '1' : '0');
	}
	return result;
}

bool Maze::operator == (const Maze &other) const {
	return fields == other.fields;
}

bool Maze::operator != (const Maze &other) const {
	return !(*this == other);
}

bool Maze::operator < (const Maze &other) const {
	return fields < other.fields;
}

