#include <deque>
#include <string>

#ifndef Maze_H_
#define Maze_H_

/**
 * Maze is represented by fields. A field can be true/false
 * whereas false represents an empty field and true represents a wall.
 */
class Maze {
protected:
	std::deque<bool> fields;
	unsigned int width;
	unsigned int height;
	unsigned int fieldCount;
public:
	Maze();

	/**
	 * Create new maze with specified dimensions.
	 * All fields are preset to be false.
	 */
	Maze(unsigned int width, unsigned int height);

	/**
	 * Create a new maze with random field values in {0,1}.
	 */
	static Maze createRandom(unsigned int width, unsigned int height);

	int getWidth() const;

	int getHeight() const;

	int getFieldCount() const;

	bool getFieldValue(unsigned int fieldNo) const;

	/**
	 * Copies all field values into the supplied boolean array.
	 * Therefore, getFieldCount() bools will be written.
	 */
	void getFieldValues(bool *);

	void setFieldValue(unsigned int fieldNo, bool value);

	void print() const;

	std::string toString() const;

	bool operator == (const Maze& other) const;

	bool operator != (const Maze& other) const;

	/**
	 * Lexicographical check of vector<bool>
	 */
	bool operator < (const Maze& other) const;

};

#endif /* Maze_H_ */
