#ifndef PARALLELBELLMANNFORD_H_
#define PARALLELBELLMANNFORD_H_

#include "BellmannFord.h"

class ParallelBellmannFord : public BellmannFord {
private:
	std::set<Vertex> entrances;
	std::set<Vertex> exits;
	int ghostEntranceCount;
	int ghostExitCount;

	void sendExitVertices(int destination);
	void sendEntranceVertices(int destination);
	void communicateGhostSizes();

	/**
	 * @param hasUpdate Did the recent iteration had an update?
	 * @return Accumulated update flag, taking communicated flag into consideration
	 */
	void communicateState();
	void receiveExitVertices(int source);
	void receiveEntranceVertices(int source);
	void createMessage(std::set<Vertex> source, int destination[]);

public:
	ParallelBellmannFord(Graph graph);

	void calculateShortestPath(std::set<Vertex> entrances, std::set<Vertex> exits);

	/**
	 * Attention! This will always return an empty
	 * vector as there is no parallel implementation
	 * for it yet.
	 */
	std::vector<Vertex> getShortestPath();
protected:
	void propagateDistances();
};

#endif /* PARALLELBELLMANNFORD_H_ */
