#include <math.h>
#include "mpi.h"
#include "gtest/gtest.h"
#include "../ga/CandidateSolution.h"
#include "../ga/ParallelFitnessResolver.h"
#include "../ga/Maze.h"
#include "../ga/ParallelMaze.h"
#include "ZigZagMaze.h"

TEST(ParallelFitnessCalculation, straightPath) {
	int processCount = MPI::COMM_WORLD.Get_size();

	int mazeWidth = 4;
	int mazeHeight = 2;
	ParallelMaze m = ParallelMaze(mazeWidth, mazeHeight * processCount);
	for (int i = 0; i < m.getFieldCount(); i += 1) {
		bool value = i % mazeWidth != 0;
		m.setFieldValue(i, value);
	}
	double fitness = ParallelFitnessResolver::getFitness(m);
	if (MPI::COMM_WORLD.Get_rank() == 0) {
		// "Master is not aware of correct fitness for straight path"
		EXPECT_EQ(mazeHeight * processCount - 1, floor(fitness + 0.5));
	}
}

TEST(ParallelFitnessCalculation, zigZag) {
	int size = MPI::COMM_WORLD.Get_size();
	int rank = MPI::COMM_WORLD.Get_rank();

	// "Test of straight path requires at least 3 processes."
	EXPECT_LE(3, size);

	Maze m = createZigZagMaze();

	double fitness = ParallelFitnessResolver::getFitness(m);
	if (rank == 0) {
		int expectedFitness = 3 * (m.getHeight() * size - 2) + 3;
		// "Master is not aware of correct fitness for straight path"
		EXPECT_EQ(expectedFitness, floor(0.5 + fitness));
	}
}
