#include "mpi.h"
#include "gtest/gtest.h"
#include "../ga/GeneticAlgorithm.h"
#include "../ga/ParallelGeneticAlgorithm.h"
#include "../ga/ParallelMaze.h"

TEST(parallelMaze, dimensions) {
	int size = MPI::COMM_WORLD.Get_size();

	int mazeWidth = 4;
	int heightPerRank = 2;
	int mazeHeight = heightPerRank * size;

	ParallelMaze m(mazeWidth, mazeHeight);
	EXPECT_EQ(mazeWidth, m.getWidth());
	EXPECT_EQ(heightPerRank, m.getHeight());
	EXPECT_EQ(heightPerRank * mazeWidth, m.getFieldCount());
}

TEST(parallelMaze, parallize) {
	int size = MPI::COMM_WORLD.Get_size();
	int rank = MPI::COMM_WORLD.Get_rank();

	int mazeWidth = 4;
	int mazeHeightPerRank = 2;

	Maze m = Maze(mazeWidth, mazeHeightPerRank * size);
	if (rank == ParallelGeneticAlgorithm::RANK_MASTER) {
		for (int i = 0; i < size; i += 1) {
			m.setFieldValue(i * mazeWidth * mazeHeightPerRank, true);
		}
	}
	ParallelMaze pm = ParallelMaze::parallize(m);
	EXPECT_EQ(mazeWidth, pm.getWidth());
	EXPECT_EQ(mazeHeightPerRank, pm.getHeight());
	for (int i = 0, max = pm.getFieldCount(); i < max; i += 1) {
		EXPECT_EQ(i == 0, pm.getFieldValue(i));
	}
	pm.print();
}

TEST(parallelMaze, gather) {
	int size = MPI::COMM_WORLD.Get_size();
	int rank = MPI::COMM_WORLD.Get_rank();

	int mazeWidth = 4;
	int mazeHeightPerRank = 2;

	ParallelMaze pm = ParallelMaze(mazeWidth, mazeHeightPerRank * size);
	pm.setFieldValue(0, true);

	Maze m = ParallelMaze::gather(pm);
	if (rank == ParallelGeneticAlgorithm::RANK_MASTER) {
		for (int i = 0, max = m.getFieldCount(); i < max; i += 1) {
			EXPECT_EQ(i % (mazeWidth * mazeHeightPerRank) == 0, m.getFieldValue(i));
		}
	}
}
