#include <math.h>
#include "../ga/Maze.h"
#include "../ga/CandidateSolution.h"
#include "gtest/gtest.h"
#include "createSampleMaze.h"

TEST(CandidateSolution, equalityOperator) {
	Maze m1 = Maze(4, 4);
	Maze m2 = Maze(4, 4);
	m2.setFieldValue(0,1);
	CandidateSolution c1 = CandidateSolution(0, m1, 3);
	CandidateSolution c11 = CandidateSolution(0, m1, 3);
	CandidateSolution c2 = CandidateSolution(0, m2, 3);
	CandidateSolution c21 = CandidateSolution(0, m2, 3);

	// "Equality failed: Candidate solutions with same maze are expected to be equal"
	EXPECT_EQ(c1, c11);
	EXPECT_EQ(c2, c21);
	// "Equality failed: Candidate solutions with different mazes are expected to be unequal"
	EXPECT_FALSE(c1 == c2);
}

TEST(CandidateSolution, inequalityOperator) {
	Maze m1 = Maze(4, 4);
	Maze m2 = Maze(4, 4);
	m2.setFieldValue(0,1);
	CandidateSolution c1 = CandidateSolution(0, m1, 3);
	CandidateSolution c11 = CandidateSolution(0, m1, 3);
	CandidateSolution c2 = CandidateSolution(0, m2, 3);
	CandidateSolution c21 = CandidateSolution(0, m2, 3);

	// "Inequality failed: Candidate solutions with different mazes are expected to be unequal"
	EXPECT_NE(c1, c2);
	// "Inequality failed: Candidate solutions with same maze are expected to be equal"
	EXPECT_FALSE(c1 != c11);
	EXPECT_FALSE(c2 != c21);
}

TEST(CandidateSolution, lessThanOperator) {
	Maze m1 = Maze(3, 3); // blank
	Maze m11 = Maze(3, 3); // unequal but same fitness
	m11.setFieldValue(0,1);
	Maze m2 = Maze(3, 3); // no path through
	m2.setFieldValue(0,1);
	m2.setFieldValue(1,1);
	m2.setFieldValue(2,1);
	Maze m3 = Maze(3, 3); // blocked straight path
	m3.setFieldValue(0,1);
	m3.setFieldValue(1,1);
	m3.setFieldValue(8,1);
	CandidateSolution c1 = CandidateSolution(0, m1, 2);
	CandidateSolution c11 = CandidateSolution(0, m1, 2);
	CandidateSolution c12= CandidateSolution(0, m11, 2);
	CandidateSolution c2 = CandidateSolution(0, m2, 0);
	CandidateSolution c3 = CandidateSolution(0, m3, 4);

	// "LessThan failed: A candidate solution is expected to be larger than itself"
	EXPECT_FALSE(c1 < c1);
	EXPECT_FALSE(c2 < c2);
	EXPECT_FALSE(c3 < c3);

	// "LessThan failed: A candidate solution with fitness is expected to be greater than others without"
	EXPECT_LT(c2, c1);
	EXPECT_FALSE(c1 < c2);
	EXPECT_LT(c2, c3);
	EXPECT_FALSE(c3 < c2);

	// "LessThan failed: The candidate solution with larger fitness is expected to be larger"
	EXPECT_LT(c1, c3);
	EXPECT_FALSE(c3 < c2);

	// "LessThan failed: Candidates solutions of different maze layouts are expected to be ordered"
	EXPECT_FALSE((c1 < c11) ^ (c11 < c1));
}
