#include "../ga/Vertex.h"
#include "../ga/Edge.h"
#include "gtest/gtest.h"

TEST(EdgeVertexRetrival, positive) {
	Vertex source = Vertex(1);
	Vertex destination = Vertex(2);
	Edge e = Edge(source, destination);

	// "Failed to retrieve source from edge"
	EXPECT_EQ(e.getSource().getId(), source.getId());
	// "Failed to retrieve destination from edge"
	EXPECT_EQ(e.getDestination().getId(), destination.getId());
}

TEST(EdgeVertexEquality, positive) {
	Vertex u = Vertex(1);
	Vertex v = Vertex(2);
	Edge e1 = Edge(u, v);
	Edge e2 = Edge(u, v);

	EXPECT_EQ(e1, e2);
}

TEST(EdgeEquality, negative) {
	Vertex u = Vertex(1);
	Vertex v = Vertex(2);
	Vertex w = Vertex(3);
	Edge e1 = Edge(u, v);
	Edge e3 = Edge(u, w);
	Edge e4 = Edge(w, v);
	Edge e5 = Edge(v, u);

	EXPECT_FALSE(e1 == e3); // "Edges with different source are expected to be not equal"
	EXPECT_FALSE(e1 == e4); // "Edges with different destination are expected to be not equal"
	EXPECT_FALSE(e1 == e5); // "Edges are expected to be not equal to their counterpart"
}

TEST(EdgeInequality, negative) {
	Vertex u = Vertex(1);
	Vertex v = Vertex(2);
	Edge e1 = Edge(u, v);
	Edge e2 = Edge(u, v);

	EXPECT_FALSE(e1 != e2);
}

TEST(EdgeInequality, positive) {
	Vertex u = Vertex(1);
	Vertex v = Vertex(2);
	Vertex w = Vertex(3);
	Edge e1 = Edge(u, v);
	Edge e3 = Edge(u, w);
	Edge e4 = Edge(w, v);
	Edge e5 = Edge(v, u);

	EXPECT_NE(e1, e3); // "Edges with different source are expected to be not equal"
	EXPECT_NE(e1, e4); // "Edges with different destination are expected to be not equal"
	EXPECT_NE(e1, e5); // "Edges are expected to be not equal to their counterpart"
}

TEST(EdgeLowerThan, positive) {
	Vertex u = Vertex(1);
	Vertex v = Vertex(2);
	Vertex w = Vertex(3);
	Edge e1 = Edge(u, v);
	Edge e2 = Edge(u, w);
	Edge e3 = Edge(v, w);

	EXPECT_LT(e1, e2); // "An edge is expected to be lower in case of same source but lower destination"
	EXPECT_LT(e2, e3); // "An edge is expected to be lower in case of lower source"
}

TEST(EdgeLowerThan, negative) {
	Vertex u = Vertex(1);
	Vertex v = Vertex(2);
	Vertex w = Vertex(3);
	Edge e1 = Edge(u, v);
	Edge e2 = Edge(u, w);
	Edge e3 = Edge(v, w);

	EXPECT_FALSE(e1 < e1); // "An edge is expected to be grater than itself"
	EXPECT_FALSE(e2 < e1); // "An edge is expected to be lower in case of same source but lower destination"
	EXPECT_FALSE(e3 < e2); // "An edge is expected to be lower in case of lower source"
}

TEST(EdgeLowerThan, counterpartBehaviour) {
	Vertex u = Vertex(1);
	Vertex v = Vertex(2);
	Edge e1 = Edge(u, v);
	Edge e4 = Edge(v, u);

	// "An edge and its counterpart cannot be bother smaller than another"
	EXPECT_FALSE(e1 < e4 && e4 < e1);
	EXPECT_FALSE(!(e1 < e4) && !(e4 < e1));
}
