#include "../ga/Maze.h"
#include "gtest/gtest.h"

TEST(Maze, fieldValueAccess) {
	Maze m = Maze(4, 4);
	EXPECT_EQ(0, m.getFieldValue(0));
	EXPECT_EQ(0, m.getFieldValue(1));
	EXPECT_EQ(0, m.getFieldValue(2));
	EXPECT_EQ(0, m.getFieldValue(3));
}

TEST(Maze, fieldValueManipulation) {
	Maze m = Maze(4, 4);
	m.setFieldValue(0,1);
	EXPECT_EQ(1, m.getFieldValue(0));
	m.setFieldValue(1,1);
	EXPECT_EQ(1, m.getFieldValue(1));
	m.setFieldValue(1,0);
	EXPECT_EQ(0, m.getFieldValue(1));
	m.setFieldValue(3,1);
	EXPECT_EQ(1, m.getFieldValue(3));
}

TEST(Maze, fieldValues) {
	Maze m = Maze(2, 2);
	m.setFieldValue(3,1);
	bool values[m.getFieldCount()];
	m.getFieldValues(values);
	EXPECT_EQ(0, values[0]);
	EXPECT_EQ(0, values[1]);
	EXPECT_EQ(0, values[2]);
	EXPECT_EQ(1, values[3]);
}

TEST(Maze, equalityOperator) {
	Maze m1 = Maze(4, 4);
	Maze m2 = Maze(4, 4);
	Maze m3 = Maze(4, 4);
	Maze m31 = Maze(4, 4);
	Maze m4 = Maze(3, 3);
	m3.setFieldValue(0,1);
	m31.setFieldValue(2,1);

	// "Failed equality: A maze is expected to be equal to itself"
	EXPECT_EQ(m1, m1);
	EXPECT_EQ(m3, m3);

	// "Failed equality: Blank mazes are expected to be equal"
	EXPECT_EQ(m1, m2);

	// "Failed equality: Mazes with different fields are expected to unequal"
	EXPECT_FALSE(m1 == m3);

	// "Failed equality: Mazes of different size are expected to be unequal"
	EXPECT_FALSE(m1 == m4);

	// "Failed equality: Mazes with different fields set are expected to be unequal"
	EXPECT_FALSE(m1 == m3);
}

TEST(Maze, inequalityOperator) {
	Maze m1 = Maze(4, 4);
	Maze m2 = Maze(4, 4);
	Maze m3 = Maze(4, 4);
	Maze m4 = Maze(3, 3);
	m3.setFieldValue(0,1);

	// "Failed inequality: A maze is expected to be equal to itself"
	EXPECT_FALSE(m1 != m1);
	EXPECT_FALSE(m3 != m3);
	// "Failed inequality: Blank mazes are expected to be equal"
	EXPECT_FALSE(m1 != m2);
	// "Failed inequality: Mazes with different fields are expected to unequal"
	EXPECT_NE(m1, m3);
	// "Failed inequality: Mazes of different size are expected to be unequal"
	EXPECT_NE(m1, m4);
}

TEST(Maze, lessThanOperator) {
	Maze m1 = Maze(4, 4);
	Maze m11 = Maze(4, 4);
	Maze m2 = Maze(4, 4);
	Maze m21 = Maze(4, 4);
	m2.setFieldValue(0,1);
	m21.setFieldValue(1,1);

	// "Failed lessThan: Mazes with same fields are unrelated"
	EXPECT_FALSE(m1 < m11);
	EXPECT_FALSE(m11 < m1);

	// "Failed lessThan: Blank maze is expected to be smaller than maze with at least one wall"
	EXPECT_LT(m1, m2);
	EXPECT_FALSE(m2 < m1);

	// "Failed lessThan: To different maze layouts are expected to be constant in deciding who is less"
	EXPECT_FALSE(m2 < m21 && m21 < m2);
}

TEST(Mate, stringRepresentation) {
	Maze m = Maze(2, 2);
	m.setFieldValue(0,1);
	m.setFieldValue(3,1);
	std::string expected ("1001");

	// "Unexpected string representation for maze"
	EXPECT_EQ(expected, m.toString());
}
