#include <vector>
#include "../ga/Maze.h"
#include "../ga/MazeToGraphConverter.h"
#include "gtest/gtest.h"
#include "createSampleMaze.h"

TEST(MazeToGraphConverter, emptyFieldBecomesVertex) {
	Maze maze = createSampleMaze(1, std::vector<int>());
	MazeToGraphConverter converter;
	converter.createGraph(maze);
	Graph g = converter.getGraph();

	// "Empty field is expected to become a vertex"
	EXPECT_TRUE(converter.hasVertexRepresentation(0));
	// "Unexpected vertex count"
	EXPECT_EQ(1, g.getVertexCount());
	// "Unexpected edge count"
	EXPECT_EQ(0, g.getEdgeCount());
}

TEST(MazeToGraphConverter, horizontalEdgeCreation) {
	std::vector<int> values = std::vector<int>();
	values.push_back(0);
	values.push_back(0);
	values.push_back(1);
	values.push_back(1);
	Maze maze = createSampleMaze(2, values);
	MazeToGraphConverter converter;
	converter.createGraph(maze);
	Graph g = converter.getGraph();

	// "Empty field is expected to become a vertex"
	EXPECT_TRUE(converter.hasVertexRepresentation(0));
	EXPECT_TRUE(converter.hasVertexRepresentation(1));

	Vertex u = converter.getVertexForField(0);
	Vertex v = converter.getVertexForField(1);

	// "Horizontal edge were not created"
	EXPECT_TRUE(g.hasEdge(u,v));
	// "Horizontal edge has wrong edge weight"
	EXPECT_EQ(1, g.getEdgeWeight(u,v));
}

TEST(MazeToGraphConverterTest, verticalEdgeCreation) {
	std::vector<int> values = std::vector<int>();
	values.push_back(0);
	values.push_back(1);
	values.push_back(0);
	values.push_back(1);
	Maze maze = createSampleMaze(2, values);
	MazeToGraphConverter converter;
	converter.createGraph(maze);

	// "Empty field is expected to become a vertex"
	EXPECT_TRUE(converter.hasVertexRepresentation(0));
	EXPECT_TRUE(converter.hasVertexRepresentation(2));

	Vertex u = converter.getVertexForField(0);
	Vertex v = converter.getVertexForField(2);
	Graph g = converter.getGraph();

	// "Vertical edge were not created"
	EXPECT_TRUE(g.hasEdge(u,v));
	// "Vertical edge has wrong edge weight"
	EXPECT_EQ(1, g.getEdgeWeight(u,v));
}

TEST(MazeToGraphConverter, entranceVertices) {
	std::vector<int> values = std::vector<int>();
	values.push_back(0);
	values.push_back(1);
	values.push_back(0);
	values.push_back(1);
	Maze maze = createSampleMaze(2, values);
	MazeToGraphConverter converter;
	converter.createGraph(maze);
	Vertex entrence = converter.getVertexForField(0);
	std::set<Vertex> entrances = converter.getEntranceVertices();

	// "The graph is expected to have 1 entrance vertex"
	EXPECT_EQ(1, entrances.size());
	// "The expected entrance was not returned"
	EXPECT_EQ(1, entrances.count(entrence));
}

TEST(MazeToGraphConverter, exitVertices) {
	std::vector<int> values = std::vector<int>();
	values.push_back(0);
	values.push_back(1);
	values.push_back(0);
	values.push_back(1);
	Maze maze = createSampleMaze(2, values);
	MazeToGraphConverter converter;
	converter.createGraph(maze);
	Vertex exit = converter.getVertexForField(2);
	std::set<Vertex> exits = converter.getExitVertices();

	// "The graph is expected to have 1 exit vertex"
	EXPECT_EQ(1, exits.size());
	// "The expected exit was not returned"
	EXPECT_EQ(1, exits.count(exit));
}
