#include "../ga/SelectionResolver.h"
#include "gtest/gtest.h"

// strongest: no straight path but path exists
CandidateSolution createStrongCandidate() {
	Maze m;
	m = Maze(3, 3);
	m.setFieldValue(0,1);
	m.setFieldValue(1,1);
	m.setFieldValue(7,1);
	m.setFieldValue(8,1);
	return CandidateSolution(0, m, 4);
}

// medium: straight path
CandidateSolution createMediumCandidate() {
	Maze m = Maze(3, 3);
	return CandidateSolution(0, m, 2);
}

CandidateSolution createWeakCandidate() {
	// weakest: no path
	Maze m = Maze(3, 3);
	m.setFieldValue(0,1);
	m.setFieldValue(1,1);
	m.setFieldValue(2,1);
	return CandidateSolution(0, m, 0);
}

TEST(SelectionResolver, removalOfTheWeakest) {
	std::set<CandidateSolution> population;
	CandidateSolution strong = createStrongCandidate();
	CandidateSolution medium = createMediumCandidate();
	CandidateSolution weak = createWeakCandidate();
	population.insert(strong);
	population.insert(medium);
	population.insert(weak);

	unsigned int remainingSize = population.size() - 1;
	SelectionResolver::elitism(population, remainingSize);

	// "Expect the remaining population to be shrunken as defined"
	EXPECT_EQ(remainingSize, population.size());

	// "Expect the strongest to remain alive"
	EXPECT_EQ(1, population.count(strong));

	// "Expect the medium to remain alive"
	EXPECT_EQ(1, population.count(medium));

	// "Expect the weak to be dead now"
	EXPECT_EQ(0, population.count(weak));
}

TEST(SelectionResolver, removalOfTheMostWeakest) {
	std::set<CandidateSolution> population;
	CandidateSolution strong = createStrongCandidate();
	CandidateSolution medium = createMediumCandidate();
	CandidateSolution weak = createWeakCandidate();
	population.insert(strong);
	population.insert(medium);
	population.insert(weak);
	unsigned int remainingSize = population.size() - 2;
	SelectionResolver::elitism(population, remainingSize);

	// "Expect the remaining population to be shrunken as defined"
	EXPECT_EQ(remainingSize, population.size());

	// "Expect the strongest to remain alive"
	EXPECT_EQ(1, population.count(strong));

	// "Expect the medium to remain alive"
	EXPECT_EQ(0, population.count(medium));

	// "Expect the weak to be dead now"
	EXPECT_EQ(0, population.count(weak));
}

