#include "../ga/SpringResolver.h"
#include "gtest/gtest.h"

Maze createHomogenousMaze(bool value) {
	Maze m = Maze(8, 8);
	if (value) {
		for (int i = 0; i < m.getFieldCount(); i += 1) {
			m.setFieldValue(i, value);
		}
	}
	return m;
}

TEST(springResolver, testSizes) {
	Maze m1 = createHomogenousMaze(0);
	Maze m2 = createHomogenousMaze(1);

	std::pair<Maze, Maze> parents (m1, m2);
	std::pair<Maze, Maze> spring = SpringResolver::createSpring(parents);

	// "Expect spring mazes to be equal width as their parents"
	EXPECT_EQ(m1.getWidth(), m2.getWidth());
	EXPECT_EQ(m1.getWidth(), spring.first.getWidth());
	EXPECT_EQ(m2.getWidth(), spring.second.getWidth());

	// "Expect spring mazes to be equal height as their parents"
	EXPECT_EQ(m1.getHeight(), m2.getHeight());
	EXPECT_EQ(m1.getHeight(), spring.first.getHeight());
	EXPECT_EQ(m2.getHeight(), spring.second.getHeight());
}

TEST(springResolver, fieldContinuity) {
	Maze m1 = createHomogenousMaze(false);
	Maze m2 = createHomogenousMaze(true);

	std::pair<Maze, Maze> parents (m1, m2);
	std::pair<Maze, Maze> spring  = SpringResolver::createSpring(parents);

	int breakPoint = -1;
	Maze child = spring.first;
	bool fieldValue;
	for (int i = 0; i < m1.getFieldCount(); i += 1) {
		fieldValue = child.getFieldValue(i);
		if (breakPoint < 0 && fieldValue) {
			breakPoint = i;
		}
		// "Expect that first child is made up by two partitions"
		EXPECT_FALSE(breakPoint >= 0 && !fieldValue);
	}
	child = spring.second;
	for (int i = 0; i < m2.getFieldCount(); i += 1) {
		fieldValue = child.getFieldValue(i);
		// "Expect that both child share a break point"
		EXPECT_FALSE(breakPoint < 0 && !fieldValue);
		// "Expect that second child is made up by first partitions up to the breakpoint"
		EXPECT_FALSE(breakPoint >= 0 && i < breakPoint && !fieldValue);
		// "Expect that second child is made up by second partition since breakpoint"
		EXPECT_FALSE(breakPoint >= 0 && i >= breakPoint && fieldValue);
	}
}
