function DataAdapter(data) {
    var that = {},
        iterations = [],
        candidates = {};
    
    var labels = {
        iteration_no: 'iteration_no',
        event_label: 'event_label',
        candidate_id: 'candidate_id',
        fitness: 'fitness',
        maze: 'maze',
        parent1_id: 'parent1_id',
        parent2_id: 'parent2_id'
    }
    
    var keywordBirth = 'BIRTH',
        keywordDeath = 'DEATH';

    /**
     * Makes use of implicit knowledge stated in the project's
     * README.md to know about the data structure present.
     */
    function dataToObjects(data) {
        data.forEach(function(entry){
            var iterationNo = entry[labels.iteration_no],
                candidateId = entry[labels.candidate_id],
                iteration,
                candidate;
            
            if (iterations.length > iterationNo) {
                iteration = iterations[iterationNo];
            } else {
                iteration = Iteration(iterationNo);
                iterations.push(iteration);
            }
            
            if (entry[labels.event_label] == keywordBirth) {
                var fitness = entry[labels.fitness],
                    maze = entry[labels.maze],
                    parentId,
                    parent;
                candidate = Candidate(candidateId, iteration, fitness, maze);
                candidates[candidateId] = candidate;
                iteration.addBirth(candidate);
                
                parentId = entry[labels.parent1_id];
                if (parentId) {
                    parent = candidates[parentId];
                    parent.addChild(candidate);
                    candidate.addParent(parent);
                }
                parentId = entry[labels.parent2_id];
                if (parentId) {
                    parent = candidates[parentId];
                    parent.addChild(candidate);
                    candidate.addParent(parent);
                }
            } else if (entry[labels.event_label] == keywordDeath) {
                candidate = candidates[candidateId];
                iteration.addDeath(candidate);
                candidate.setDeath(iteration);
            } else {
                throw new Error("Unknown event in log: " + entry[labels.event_label]);
            }
        });
    }
    
    function getIterations() {
        return iterations;
    }
    
    function getCandidates() {
        return candidates;
    }
    
    dataToObjects(data);
    that.getIterations = getIterations;
    that.getCandidates = getCandidates;
    return that;
    
}